

#include <sys/types.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "Tachyon.h"

const int MAX_MESSAGE_SIZE=100;

static void message_notification(union sigval note)
{
   Tachyon *tp = (Tachyon *) note.sival_ptr;
   tp->receive_message();
}

Tachyon::Tachyon()
{
   offset = 0;
   T0     = 0;

   accelleration = 1.0;

   struct mq_attr  queue_attributes;
   struct sigevent sev;

   sprintf(queue_name, "/Tachyon.%d", getpid());
   queue_attributes.mq_maxmsg = 2;
   queue_attributes.mq_msgsize = MAX_MESSAGE_SIZE;

   msg_queue = mq_open(queue_name, O_CREAT | O_RDONLY, S_IRWXU, &queue_attributes);

   sev.sigev_notify = SIGEV_THREAD;
   sev.sigev_notify_function = message_notification;
   sev.sigev_notify_attributes = NULL;
   sev.sigev_value.sival_ptr = this;  
   mq_notify(msg_queue, &sev);
}

Tachyon::Tachyon(const char *name)
{
   offset = 0;
   T0     = 0;

   accelleration = 1.0;

   queue_name[0] = '\0';
   msg_queue = mq_open(name, O_WRONLY);
}

Tachyon::~Tachyon()
{
   mq_close(msg_queue);
   if (queue_name[0] != '\0')
   {
      mq_unlink(queue_name);
   }
}

const char * Tachyon::name(void)
{
   return queue_name;
}

void Tachyon::receive_message(void)
{
   char message[MAX_MESSAGE_SIZE];
   int  message_size;

   printf("Receiving message.\n");
   message_size = mq_receive(msg_queue, message, MAX_MESSAGE_SIZE, NULL);
   message[message_size] = '\0';

   printf("The message is %s.\n", message);
   switch (message[0])
   {
   case 'A':
      accellerate(atof(message+1));
      break;

   default:
      // Invalid message.
      break;
   }
}

time_t Tachyon::time(void)
{
}

int Tachyon::nanosleep(struct timespec req)
{
   double seconds;
   double nanoseconds;

   seconds     = req.tv_sec  / accelleration;
   nanoseconds = req.tv_nsec / accelleration;
   nanoseconds += (seconds - trunc(seconds)) * 1.0e9;
   req.tv_sec  = trunc(seconds);
   req.tv_nsec = trunc(nanoseconds);

   ::nanosleep(&req, NULL);
}

void Tachyon::settime(time_t sec)
{
}

void Tachyon::accellerate(double factor)
{
   accelleration = factor;

   //  send to the Tachyon object in another process.
   if (queue_name[0] == '\0')
   {
      char message[MAX_MESSAGE_SIZE];

      sprintf(message, "A%f", accelleration);
      mq_send(msg_queue, message, strlen(message), 100);
   }
}
